@@ -41,8 +41,7 @@
 #include "version.h"
 #include "net-support.h"
 #include "pathnames.h"
-#define  EXTERN
-#include "net-locale.h"
+#include "intl.h"
 
 extern int h_errno;  /* some netdb.h versions don't export this */
 
@@ -120,7 +119,7 @@ INET_rresolve(char *name, struct sockaddr_in *sin, int numeric)
   /* Grmpf. -FvK */
   if (sin->sin_family != AF_INET) {
 #ifdef DEBUG
-	fprintf(stderr, NLS_CATGETS(catfd, inetSet, inet_debug1, "rresolve: unsupport address family %d !\n"), sin->sin_family);
+	fprintf(stderr, _("rresolve: unsupport address family %d !\n"), sin->sin_family);
 #endif
 	errno = EAFNOSUPPORT;
 	return(-1);
@@ -200,7 +199,7 @@ INET_sprint(struct sockaddr *sap, int numeric)
   static char buff[128];
 
   if (sap->sa_family == 0xFFFF || sap->sa_family == 0)
-    return(NLS_CATBUFF (catfd, inetSet, inet_none, "[NONE SET]", buff, 128));
+	return strncpy (buff, _("[NONE SET]"), sizeof(buff));
   if (INET_rresolve(buff, (struct sockaddr_in *) sap, numeric) != 0)
 							return(NULL);
   return(buff);
@@ -211,7 +210,7 @@ static int
 INET_getsock(char *bufp, struct sockaddr *sap)
 {
   char *sp = bufp,*bp;
-  int i;
+  unsigned int i;
   unsigned val;
   struct sockaddr_in *sin;
   
@@ -260,11 +259,35 @@ INET_input(int type, char *bufp, struct sockaddr *sap)
   }
 }
 
+static int INET_getnetmask(char *adr, struct sockaddr *m, char *name)
+{ 
+	struct sockaddr_in *mask = (struct sockaddr_in *)m; 
+	char *slash, *end; 
+	int prefix; 
+
+	if ((slash = strchr(adr, '/')) == NULL) 
+		return 0; 
+		
+	*slash++ = '\0';
+	prefix = strtoul(slash,&end,0);
+	if (*end != '\0') 
+		return -1; 
+
+	if (name) {
+		sprintf(name, "/%d", prefix); 
+	}
+		
+	mask->sin_family = AF_INET; 
+    mask->sin_addr.s_addr = htonl(~(0xffffffffU >> prefix)); 
+	return 1;
+} 
+
 
 struct aftype inet_aftype = {
   "inet",	NULL, /*"DARPA Internet",*/	AF_INET,	sizeof(unsigned long),
   INET_print,	INET_sprint,		INET_input,	INET_reserror,	
-  NULL/*INET_rprint*/,	NULL/*INET_rinput*/
+  NULL/*INET_rprint*/,	NULL/*INET_rinput*/,
+  INET_getnetmask 
 };
 
 #endif	/* HAVE_AFINET || HAVE_AFINET6 */
